class CameraViewController {
    static cameraPrivilege = "http://tizen.org/privilege/camera"

    constructor(view) {
        this.view = view
        this.canvas = view.querySelector("#camera-output")
        this.ctx = this.canvas.getContext("2d")
        this.canvasProcessed = view.querySelector("#processed-output")
        this.canvasProcessed.width = 28
        this.canvasProcessed.height = 28
        this.ctxProcessed = this.canvasProcessed.getContext("2d")
        this.ctxProcessed.filter = "grayscale(1) invert(1) contrast(1.5)"
        this.switchCameraButton = view.querySelector("#switch-camera-button")
        this.switchViewButton = view.querySelector("#switch-view-button")
        this.modelOutput = view.querySelector(".model-output")
        this.cameras = []
        this.currentCameraIndex = null
        this.stream = null
        this.inputTensorInfo = new tizen.ml.TensorsInfo()
        this.inputTensorInfo.addTensorInfo("tensor", "FLOAT32", [1, 28, 28, 1])
        this.canvasInterval = null
        this.predictInterval = null

        if(MnistModel.instance.duringTraining) {
            this.modelOutput.innerHTML = "<span>Training the model...</span>"
        }

        try {
            this.startCamera()
        }
        catch(error) {
            Logger.instance.error(error)
        }
    }

    async startCamera() {
        if(await requestPermission(CameraViewController.cameraPrivilege) === false) {
            Logger.instance.info("Camera access denied")
            return
        }

        const devices = await window.navigator.mediaDevices.enumerateDevices()
        this.cameras = devices.filter(d => d.kind === "videoinput").map(d => d.deviceId)

        if(this.cameras.length > 0) {
            this.currentCameraIndex = 0
            this.changeCamera(this.currentCameraIndex)
            this.switchCameraButton.disabled = false
            this.switchViewButton.disabled = false

            this.switchCameraButton.addEventListener("click", () => {
                this.currentCameraIndex = (this.currentCameraIndex + 1) % this.cameras.length
                this.changeCamera(this.currentCameraIndex)
            })

            this.switchViewButton.addEventListener("click", () => {
                if(this.canvas.hidden) {
                    this.canvas.hidden = false
                    this.canvasProcessed.hidden = true
                    this.switchViewButton.innerHTML = "Processed image"
                }
                else {
                    this.canvas.hidden = true
                    this.canvasProcessed.hidden = false
                    this.switchViewButton.innerHTML = "Raw image"
                }
            })
        }
        else {
            Logger.instance.error("No cameras")
        }
    }

    stopCamera() {
        if(this.predictInterval) {
            clearInterval(this.predictInterval)
            this.predictInterval = null
        }
        if(this.canvasInterval) {
            clearInterval(this.canvasInterval)
            this.canvasInterval = null
        }
        if(this.stream) {
            this.stream.getTracks().forEach(t => t.stop())
            this.stream = null
        }
    }

    changeCamera(index) {
        this.stopCamera()
        const config = {
            video: {
                deviceId: {
                    exact: this.cameras[index]
                }
            },
            audio: false,
        }
        window.navigator.webkitGetUserMedia(
            config,
            stream => {
                this.stream = stream
                const video = document.createElement("video")
                video.srcObject = stream
                video.addEventListener("loadedmetadata", () => {
                    video.play()
                    const size = Math.min(video.videoWidth, video.videoHeight)
                    this.canvas.width = size
                    this.canvas.height = size
                    const sx = (video.videoWidth - size) / 2
                    const sy = (video.videoHeight - size) / 2
                    this.canvasInterval = setInterval(() => {
                        this.ctx.drawImage(video, sx, sy, size, size, 0, 0, size, size)
                        this.ctxProcessed.drawImage(video, sx, sy, size, size, 0, 0, 28, 28)
                    }, 1000 / 30)
                    this.predictInterval = setInterval(this.predict.bind(this), 2000)
                })
            },
            error => {
                Logger.instance.error(error)
            }
        )
    }

    async predict() {
        if(MnistModel.instance.duringTraining) {
            return
        }

        try {
            const imageData = this.ctxProcessed.getImageData(0, 0, 28, 28)
            let data = imageData.data.filter((_, i) => i % 4 == 0)
            const a = Math.min(...data)
            const b = Math.max(...data) - a
            data = data.map(v => (v - a) / b)
            const tensorData = this.inputTensorInfo.getTensorsData()
            tensorData.setTensorRawData(0, new Float32Array(data))
            const y = await MnistModel.instance.predict(tensorData)
            tensorData.dispose()
            const result = processModelOutput(y)
            this.modelOutput.innerHTML = ""
            result.forEach(([i, v]) => {
                this.modelOutput.innerHTML += `
                    <div class="result">
                        <span class="num">${i}</span>
                        <span class="prop">${(v * 100).toFixed(2)}%</span>
                    </div>`
            })
        }
        catch(error) {
            Logger.instance.error(`model predict error: ${error}`)
        }
    }

    dispose() {
        this.stopCamera()
        this.inputTensorInfo.dispose()
    }
}

