class DrawViewController {
    constructor(view) {
        this.view = view
        this.canvas = view.querySelector("#canvas-input")
        this.clearButton = view.querySelector("#canvas-clear-button")
        this.modelOutput = view.querySelector(".model-output")
        this.ctx = this.canvas.getContext("2d")
        this.ctx.imageSmoothingEnabled = false
        this.canvas.width = 28
        this.canvas.height = 28
        this.drawing = false
        this.predictInterval = null
        this.inputTensorInfo = new tizen.ml.TensorsInfo()
        this.inputTensorInfo.addTensorInfo("tensor", "FLOAT32", [1, 28, 28, 1])
        this.ongoingTouches = {}
        this.clearCanvas()

        this.canvas.addEventListener("touchstart", this.onTouchStart.bind(this))
        this.canvas.addEventListener("touchend", this.onTouchEnd.bind(this))
        this.canvas.addEventListener("touchcancel", this.onTouchCancel.bind(this))
        this.canvas.addEventListener("touchmove", this.onTouchMove.bind(this))
        this.clearButton.addEventListener("click", this.clearCanvas.bind(this))

        if(MnistModel.instance.duringTraining) {
            this.modelOutput.innerHTML = "<span>Training the model...</span>"
        }
    }

    onTouchStart(event) {
        Object.entries(event.changedTouches).forEach(([_, touch]) => {
            const rect = this.canvas.getBoundingClientRect()
            const x = (touch.clientX - rect.left) / rect.width * this.canvas.width
            const y = (touch.clientY - rect.top) / rect.height * this.canvas.height
            this.ctx.fillStyle = "white"
            this.ctx.fillRect(x - 1, y - 1, 2, 2)
            this.ongoingTouches[touch.identifier] = { x: x, y: y }
        })
    }

    onTouchMove(event) {
        Object.entries(event.changedTouches).forEach(([_, touch]) => {
            if(this.ongoingTouches[touch.identifier]) {
                const rect = this.canvas.getBoundingClientRect()
                const x = (touch.clientX - rect.left) / rect.width * this.canvas.width
                const y = (touch.clientY - rect.top) / rect.height * this.canvas.height
                this.ctx.beginPath()
                this.ctx.moveTo(this.ongoingTouches[touch.identifier].x, this.ongoingTouches[touch.identifier].y)
                this.ctx.lineTo(x, y)
                this.ctx.lineWidth = 2
                this.ctx.strokeStyle = "white"
                this.ctx.stroke()
                this.ongoingTouches[touch.identifier].x = x
                this.ongoingTouches[touch.identifier].y = y
            }
        })
    }

    onTouchEnd(event) {
        this.onTouchMove(event)
        Object.entries(event.changedTouches).forEach(([_, touch]) => {
            this.ongoingTouches[touch.identifier] = undefined
        })
        this.predict()
    }

    onTouchCancel(event) {
        Object.entries(event.changedTouches).forEach(([_, touch]) => {
            this.ongoingTouches[touch.identifier] = undefined
        })
        this.predict()
    }

    clearCanvas() {
        this.ctx.fillStyle = "black"
        this.ctx.fillRect(0, 0, this.canvas.width, this.canvas.height)

        if(MnistModel.instance.duringTraining) {
            this.modelOutput.innerHTML = "<span>Training the model...</span>"
        }
        else {
            this.modelOutput.innerHTML = "<span>Draw a digit.</span>"
        }
    }

    async predict() {
        if(MnistModel.instance.duringTraining) {
            return
        }

        try {
            const imageData = this.ctx.getImageData(0, 0, this.canvas.width, this.canvas.height)
            const data = imageData.data.filter((_, i) => i % 4 == 0).map(v => v / 255.0)
            const tensorData = this.inputTensorInfo.getTensorsData()
            tensorData.setTensorRawData(0, new Float32Array(data))
            const y = await MnistModel.instance.predict(tensorData)
            tensorData.dispose()
            const result = processModelOutput(y)
            this.modelOutput.innerHTML = ""
            result.forEach(([i, v]) => {
                this.modelOutput.innerHTML += `
                    <div class="result">
                        <span class="num">${i}</span>
                        <span class="prop">${(v * 100).toFixed(2)}%</span>
                    </div>`
            })
        }
        catch(error) {
            Logger.instance.error(`model predict error: ${error}`)
        }
    }

    dispose() {
        this.inputTensorInfo.dispose()
        this.inputTensorInfo = null
        if(this.predictInterval) {
            clearInterval(this.predictInterval)
            this.predictInterval = null
        }
    }
}
